package medianRanking;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import utils.Wrappers;
import medianRanking.KendallTauGenerlized.KendallTauType;

/**
 * 
 * @author brancotte
 * 
 */
public class KendallTauGeneralizedTree {

	protected static class BalancedTreeBucket<T> {
		protected Collection<T> bucket;
		protected BalancedTreeBucket<T> before, after, parentAfter, parentBefore;
		protected int mid;
		protected int size;

		public BalancedTreeBucket(List<Collection<T>> ranking) {
			this(ranking, 0, ranking.size() - 1, null, null);
		}

		public BalancedTreeBucket(List<Collection<T>> ranking, int from, int to, BalancedTreeBucket<T> parentBefore,
				BalancedTreeBucket<T> parentAfter) {
			mid = ((to - from) >> 1) + from;
			bucket = ranking.get(mid);
			this.size = bucket.size();
			this.parentAfter = parentAfter;
			this.parentBefore = parentBefore;
			if (from < mid) {
				before = new BalancedTreeBucket<T>(ranking, from, mid - 1, null, this);
				size += before.size;
			}
			if (mid < to) {
				after = new BalancedTreeBucket<T>(ranking, mid + 1, to, this, null);
				size += after.size;
			}
		}

		public BalancedTreeBucket<T> get(T a) {
			if (bucket.contains(a))
				return this;
			BalancedTreeBucket<T> btb;
			if (before != null && null != (btb = before.get(a)))
				return btb;
			if (after != null)
				return after.get(a);
			return null;
		}

		public int getOrder(T a) {
			if (bucket.contains(a))
				return 0;
			if (hasBefore(a))
				return -1;
			if (hasAfter(a))
				return 1;
			return 2;
		}

		public Iterator<T> iterator() {
			return new Iterator<T>() {

				@Override
				public boolean hasNext() {
					// TODO Auto-generated method stub
					return false;
				}

				@Override
				public T next() {
					// TODO Auto-generated method stub
					return null;
				}

				@Override
				public void remove() {
				}
			};
		}

		public boolean hasBefore(T a) {
			// return (before != null && before.hasChild(a))
			// || (parentBefore != null && (parentBefore.bucket.contains(a) ||
			// parentBefore.hasBefore(a)));
			return before != null && before.hasChild(a) || parentBefore != null
					&& (parentBefore.has(a) || parentBefore.before != null && parentBefore.before.hasChild(a));
		}

		public boolean hasAfter(T a) {
			return after != null && after.hasChild(a) || parentAfter != null
					&& (parentAfter.has(a) || parentAfter.after != null && parentAfter.after.hasChild(a));
		}

		private boolean hasChild(T a) {
			return has(a) || before != null && before.hasChild(a) || after != null && after.hasChild(a);
		}

		// public boolean hasAfter(T a) {
		// return (after != null && (after.bucket.contains(a) ||
		// after.hasAfter(a)))
		// || (parentAfter != null && (parentAfter.bucket.contains(a) ||
		// parentAfter.hasAfter(a)));
		// }

		public boolean has(T a) {
			return bucket.contains(a);
		}

		public String toString() {
			StringBuilder sb = new StringBuilder();
			// sb.append('[');
			toString(sb, true);
			// sb.append(']');
			return sb.toString();
		}

		public String toString(StringBuilder sb, boolean tree) {
			if (before != null) {
				if (tree)
					sb.append('[');
				if (tree)
					sb.append('£');
				before.toString(sb, tree);
				if (tree)
					sb.append(']');
				if (!tree)
					sb.append(',');
			}
			sb.append(bucket);
			if (after != null) {
				if (tree)
					sb.append('[');
				if (tree)
					sb.append('$');
				if (!tree)
					sb.append(',');
				after.toString(sb, tree);
				if (tree)
					sb.append(']');
			}
			return sb.toString();
		}
	}

	public <T> Map<KendallTauType, Float> evaluateResults(List<Collection<T>> c, Collection<List<Collection<T>>> R) {

		return null;
	}

	public static void main(String[] args) {
		BalancedTreeBucket<String> r;
		List<Collection<String>> ranking;
		r = new BalancedTreeBucket<String>(MedianRankingTools.toRankingWithTie("[[g],[a],[f],[b,c],[d],[e],[h,i]]"));
		System.out.println(r);
		r = new BalancedTreeBucket<String>(ranking = MedianRankingTools.toRankingWithTie("[[a],[b],[c],[d],[e],[f],[g],[h,i],[j],[k],[l]]"));
		System.out.println(r);
		System.out.println(r);
		System.out.println(r.toString(new StringBuilder(), true));
		Set<String> elts = MedianRankingTools.getDistinctElement(Wrappers.wrapInCollection(ranking));
		for (String a : elts) {
			for (String b : elts) {
				if (!a.equals(b)) {
					System.out.print(a + " hasAfter " + b + " :");
					System.out.println(r.get(a).hasAfter(b));
					System.out.print(b + " hasAfter " + a + " :");
					System.out.println(r.get(b).hasAfter(a));
				}
			}
		}
		// System.out.println(r.get("e").hasAfter("a"));
		// System.out.println(r.get("e").hasAfter("f"));
		// System.out.println(r.get("e").hasAfter("b"));
		// System.out.println(r.get("e").hasAfter("g"));
		// System.out.println(r.get("a").hasAfter("f"));
		// System.out.println(r.get("a").hasBefore("f"));

	}

}
