package medianRanking;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

public class KendallTauGenerlizedNSquare implements KendallTauGenerlized {

	public static KendallTauGenerlizedNSquare instance = new KendallTauGenerlizedNSquare();

	private float p;
	private float q;

	public KendallTauGenerlizedNSquare() {
		this(KendallTauType.Generalized);
	}

	public KendallTauGenerlizedNSquare(KendallTauType type) {
		this(type == KendallTauType.Generalized ? 1F : 0F);
	}

	/**
	 * 
	 * @param p
	 *            cost when a pair is in the same bucket in one ranking and not
	 *            in the same in the other ranking
	 */
	public KendallTauGenerlizedNSquare(float p) {
		this(p, p);
	}

	/**
	 * @param p
	 *            cost when a pair is in the same bucket in one ranking and not
	 *            in the same in the other ranking
	 * @param q
	 *            cost when two element are not in the gold standard, and in
	 *            different bucket in the results
	 */
	public KendallTauGenerlizedNSquare(float p, float q) {
		this.p = p;
		this.q = q;
	}

	public <T> Map<KendallTauType, Float> evaluateResults(List<Collection<T>> elementsR1, List<Collection<T>> elementsR2) {
		Map<T, int[]> rankElt = new HashMap<T, int[]>();
		int level = 0;
		int dst = 0, dstP = 0, dstQ = 0, agree = 0;
		for (Collection<T> bucket : elementsR1) {
			for (T elt : bucket) {
				rankElt.put(elt, new int[] { level, -1 });
			}
			level++;
		}
		level = 0;
		for (Collection<T> bucket : elementsR2) {
			for (T elt : bucket) {
				int[] lvl = rankElt.get(elt);
				if (lvl == null) {
					rankElt.put(elt, new int[] { -1, level });
				} else
					lvl[1] = level;
			}
			level++;
		}
		// List<int[]> elements = new Vector<int[]>(rankElt.values());
		List<Entry<T, int[]>> elements = new Vector<Entry<T, int[]>>(rankElt.entrySet());
		rankElt.entrySet();
		for (int a = elements.size() - 1; a > 0; a--) {
			// int[] posA = elements.get(a);
			Entry<T, int[]> posA = elements.get(a);
			if (posA.getValue()[0] != -1 && posA.getValue()[1] != -1) {
				for (int b = a - 1; b >= 0; b--) {
					// int[] posB = elements.get(a);
					Entry<T, int[]> posB = elements.get(b);
					if (posB.getValue()[0] != -1 && posB.getValue()[1] != -1) {
						int sngR1 = Integer.signum(posA.getValue()[0] - posB.getValue()[0]);
						int sngR2 = Integer.signum(posA.getValue()[1] - posB.getValue()[1]);
						if (sngR1 != sngR2) {
							if (sngR1 == 0) {
								dstQ++;
							} else if (sngR2 == 0) {
								dstP++;
							} else {
								dst++;
							}
						} else {
							agree++;
						}
					}
				}
			}
		}
		Map<KendallTauType, Float> map;
		map = new HashMap<KendallTauType, Float>(4);
		map.put(KendallTauType.OriginalKen38, new Float(dst));
		map.put(KendallTauType.Generalized, dst + p * (dstP + dstQ));
		map.put(KendallTauType.Asymetric, dst + p * dstP + q * dstQ);
		float disagree = dst + p * dstP + q * dstQ;
		map.put(KendallTauType.SimilarityMeasure, (agree - disagree) / (agree + disagree));
		return map;
	}

	@Override
	public <T> Map<KendallTauType, Float> evaluateResults(List<Collection<T>> c, Collection<List<Collection<T>>> inputRankings) {
		float o = 0, g = 0, a = 0;
		for (List<Collection<T>> r : inputRankings) {
			Map<KendallTauType, Float> m = evaluateResults(c, r);
			o += m.get(KendallTauType.OriginalKen38);
			g += m.get(KendallTauType.Generalized);
			a += m.get(KendallTauType.Asymetric);

		}
		Map<KendallTauType, Float> map;
		map = new HashMap<KendallTauType, Float>(4);
		map.put(KendallTauType.OriginalKen38, o);
		map.put(KendallTauType.Generalized, g);
		map.put(KendallTauType.Asymetric, a);
		map.put(KendallTauType.SimilarityMeasure, Float.NaN);
		return map;
	}
}
